import os
from diffusers import StableDiffusionPipeline, DDIMScheduler
import torch

from tqdm import tqdm, trange


import torch
from safetensors.torch import load_file, save_file
from diffusers import StableDiffusionPipeline
from diffusers import DPMSolverMultistepScheduler

from diffusers.loaders.lora import LoraLoaderMixin


from inference_configs import parse_args

def main():
    opt = parse_args()

    scheduler = DDIMScheduler.from_pretrained(opt.pretrained_model_name_or_path, subfolder="scheduler")
    pipe = StableDiffusionPipeline.from_pretrained(opt.pretrained_model_name_or_path, scheduler=scheduler, torch_dtype=torch.float32
    )

    #####################################################################################
    if opt.dora_ckpt_dir is not None:
        state_dict, network_alphas = pipe.lora_state_dict(
            opt.dora_ckpt_dir,
            unet_config=pipe.unet.config,
        )
       

        is_correct_format = all("lora" in key or "dora_scale" in key for key in state_dict.keys())
        if not is_correct_format:
            raise ValueError("Invalid DoRA checkpoint.")
       

        #####################################################################################

        LoraLoaderMixin.load_lora_into_unet(
            state_dict = state_dict,
            network_alphas = network_alphas, 
            unet = pipe.unet,
        )

        print('load DoRA checkpoint: done!')

    pipe = pipe.to("cuda")


    print(f"reading prompts from {opt.from_file}")
    with open(opt.from_file, "r") as f:
        data = f.read().splitlines()
        data = [d.strip("\n").split("\t")[0] for d in data] 
        data = [opt.batch_size * [d] for d in data]

 
    # create output folder
    outpath = opt.outdir 
    os.makedirs(outpath, exist_ok=True)
    sample_path = os.path.join(outpath, f"samples")
    os.makedirs(sample_path, exist_ok=True)
    base_count = len(os.listdir(sample_path))
   
    # run inference
    with torch.no_grad():
        for prompts in tqdm(data, desc="data"):
            
            seed = 42
            generator = torch.Generator(device="cpu").manual_seed(seed)

            for n in trange(opt.n_iter, desc="Sampling"):
                image = pipe(prompt=prompts,
                                num_inference_steps=opt.num_inference_steps, 
                                guidance_scale=7.5,
                                generator=generator,
                                ).images

                seed += 1
                generator = torch.Generator(device="cpu").manual_seed(seed)
                
                print('len(image)', len(image))
                for i in range(len(image)):
                    image[i].save(os.path.join(sample_path, f"{prompts[0]}_{base_count:06}.png"))
                    base_count += 1

if __name__ == "__main__":
    main()
