import argparse
import os


def parse_args_finetuning():
    parser = argparse.ArgumentParser(description="Simple example of a training script.")

    ##################################################################################################
    # validation

    parser.add_argument(
        "--validation_prompts",
        type=str,
        default=None,
        nargs="+",
        help="A set of prompts evaluated every `--validation_steps` and logged to `--report_to`.",
    )
    parser.add_argument(
        "--validation_prompts_file",
        type=str,
        default='val_visu.txt',
        help="A set of prompts evaluated every `--validation_steps` and logged to `--report_to`.",
    )

    parser.add_argument(
        "--num_validation_images",
        type=int,
        default=3,
        help="Number of images that should be generated during validation for each validation prompt.",
    )
    parser.add_argument(
        "--validation_steps", 
        type=int,
        default=5,
        help="Run fine-tuning validation every X steps.",
    )

    ##################################################################################################
    # dora

    parser.add_argument(
        "--use_dora",
        action="store_true",
        default=True,
    )
    parser.add_argument(
        "--rank",
        type=int,
        default=32,
        help=("The dimension of the DoRA update matrices."),
    )
    parser.add_argument(
        "--alpha_div_rank",
        type=int,
        default=1,
        help=("The ratio alpha/rank for DoRA."),
    )

    ##################################################################################################
    # dataset 
    parser.add_argument(
        "--train_data_dir",
        type=str,
        default='../MITUNE_data',
        help=(
            "A folder containing the training data generated by ../construct_finetuning_dataset/gen_imgs_save_best_1phase.py "
        ),
    )
    parser.add_argument(
        "--max_train_samples",
        type=int,
        default=None,
        help=(
            "For debugging purposes or quicker training, truncate the number of training examples to this "
            "value if set."
        ),
    )

    ##################################################################################################
    # pre-processing data

    parser.add_argument(
        "--resolution",
        type=int,
        default=512,
        help=(
            "The resolution for input images, all the images in the train/validation dataset will be resized to this"
            " resolution"
        ),
    )
    parser.add_argument(
        "--center_crop",
        default=False,
        action="store_true",
        help=(
            "Whether to center crop the input images to the resolution. If not set, the images will be randomly"
            " cropped. The images will be resized to the resolution first before cropping."
        ),
    )
    parser.add_argument(
        "--dataloader_num_workers",
        type=int,
        default=0,
        help=(
            "Number of subprocesses to use for data loading. 0 means that the data will be loaded in the main process."
        ),
    )

    ##################################################################################################
    # train steps & batch size

    parser.add_argument( "--train_batch_size", type=int, default=20, 
                        help="Batch size (per device) for the training dataloader."
    )
    parser.add_argument( "--gradient_accumulation_steps", type=int, default=20,
        help="Number of updates steps to accumulate before performing a backward/update pass.",
    )

    parser.add_argument("--max_train_steps", type=int, default=300,
        help="Total number of training steps to perform.",
    )

    ##################################################################################################
    # sd version & seed

    parser.add_argument(
        "--pretrained_model_name_or_path",  
        type=str,
        default='stabilityai/stable-diffusion-2-1-base',
        help="Path to pretrained model or model identifier from huggingface.co/models.",
    )

    parser.add_argument("--seed", type=int, default=42, help="A seed for reproducible training.")

    ##################################################################################################
    # log

    parser.add_argument(
        "--outdir",
        type=str,
        default="../MITUNE_ckpts",
        help="The output directory where the model predictions and checkpoints will be written.",
    )

    parser.add_argument(
        "--checkpoints_total_limit",
        type=int,
        default=100,
        help="Max number of checkpoints to store.",
    )

    ##################################################################################################
    # lr & optimizer, earlyts

    parser.add_argument("--earlyts", default=True, action="store_true", )

    parser.add_argument("--learning_rate", type=float, default=1e-4, help="Initial learning rate (after the potential warmup period) to use.",  )

    parser.add_argument("--adam_beta1", type=float, default=0.9, help="The beta1 parameter for the Adam optimizer.")
    parser.add_argument("--adam_beta2", type=float, default=0.999, help="The beta2 parameter for the Adam optimizer.")
    parser.add_argument("--adam_weight_decay", type=float, default=1e-2, help="Weight decay to use.")
    parser.add_argument("--adam_epsilon", type=float, default=1e-08, help="Epsilon value for the Adam optimizer")
    
    parser.add_argument("--max_grad_norm", default=1.0, type=float, help="Max gradient norm.")

   
  
    
    parser.add_argument("--local_rank", type=int, default=-1, help="For distributed training: local_rank")




    args = parser.parse_args()

    env_local_rank = int(os.environ.get("LOCAL_RANK", -1))
    if env_local_rank != -1 and env_local_rank != args.local_rank:
        args.local_rank = env_local_rank

    if args.validation_prompts_file is not None:
        with open(args.validation_prompts_file, 'r') as f:
            val_prompts_from_file = f.read().splitlines()
        val_prompts_from_file = [d.strip("\n").split("\t")[0] for d in val_prompts_from_file] 
        args.validation_prompts = args.validation_prompts + val_prompts_from_file if args.validation_prompts is not None else val_prompts_from_file
    print(f'{len(args.validation_prompts)} validation prompts')
    
    return args
